/********************************************************************
 * GameBoard.java - GameBoard object for ConnectN client.	    *
 * Sam Braff (hebrewtoyou@hotmail.com)				    *
 ********************************************************************/

import java.lang.*;
class GameBoard {		
    
    /************************************************************
     * Structure of the gameboard - two dimensional array	*
     * representing the board configuration.		        *
     ************************************************************/

    private char matrix[][];
    private int width;
    private int height;
    private int goal;
    private int numInARow = 0;

    /************************************************************
     * Constructor						*
     * width corresponds to the number of columns    	        *
     * height corresponds to the number of rows		        *
     * goal corresponds to the connection requirement	        *
     ************************************************************/

    GameBoard(int horizontal, int vertical, int connect) {	
	this.width = horizontal;
    	this.height = vertical;
    	this.matrix = new char[width][height];
    	this.goal = connect;
    	this.empty();
    }
    /************************************************************
     * getPosition returns the contents of a row,col index in the 
     * matrix.
     ************************************************************/
    char getPosition(int col, int row) {
	return(this.matrix[col][row]);
    }


    void setPosition(int col, int row, char val) {
	this.matrix[col][row] = val;
    }

    /************************************************************
     * int getHeight()					        *
     * Returns the height [number of rows] of the board as an   *
     * integer.						        *
     ************************************************************/   
    int getHeight() {	
	return(this.height);	
    }
    
    /************************************************************ 
     * int getWidth()                                           *
     * Returns the width [number of columns] of the board as an *
     * integer.                                                 *
     ************************************************************/   
    int getWidth() {	
	return(this.width);	
    }
    
    /**
     * getGoal returns how many pieces are needed in a row to 
     * constitute a win 
     **/
    int getGoal() {
	return(this.goal);      
    }

    
    /************************************************************
     * boolean execute()					*
     * Returns false if column is full			        *
     * Returns true if move was executed			*
     * col represents the columns from 0 to (width - 1)	        *
     * Looks for an open spot in the row for a piece, and places*
     * it if available.					        *
     ************************************************************/
    boolean execute(char player, int col) {	
	int index = this.height - 1;;
	while(index >= 0) {	
	    if(this.matrix[col][index] == 'e') {	
		this.matrix[col][index] = player;
		return true;
	    }
	    else { index--; }
	}
	return false;
    }
    
    /************************************************************
     * void empty()						*
     * Sets the board to the empty state.			*
     ************************************************************/   
    void empty() {	
	int rise, count;
	for(count = 0; count < this.width; count++) {	
	    for(rise = 0; rise < this.height; rise++) {	
		this.matrix[count][rise] = 'e';	
	    }
	}
    }
    
    /************************************************************
     * boolean isDraw()                                         *
     * Returns true if the board is full and there is no winner.*
     ************************************************************/
    public boolean isDraw() {
	int i = 0;
	
	// check if the top slot in any column is empty. if so, 
	// return false;
	for(i=0;i<this.width; i++) {
	    if(this.matrix[i][0] == 'e') { return false; }
	}

	// we've validated that the board is full, but someone 
	// could have won on the final piece placement. check to
	// make sure there are no winners.
	if(isWin('r')) { return false; }
	if(isWin('b')) { return false; }

	// the board is full and no one is a winner. return true.
	else { return true; }
    }
    
    
    /************************************************************
     * boolean isWin()                                          *
     * Accepts character input representing the player          *
     * Returns true if goal has been achieved by said player    *
     ************************************************************/   
    public boolean isWin(char player) {	
	int RESET=0;
	boolean isAWin = false;
	
	for(int i=0; i<this.width; i++) {	
	    for(int j=0; j<this.height; j++) {	
		numInARow = 0;
	    	
		if((this.matrix[i][j] == player)) {
	
		    // Tests the left-down diagonal	
		    if((i!=0) && (j != this.height-1)) {	
			isAWin = testDownLeft(i,j,player);

			if(isAWin) { return true; }
			else {	numInARow = RESET;	}
		    }
		    
		    // Tests the vertical		    
		    if(j<this.height-1)	{	
			isAWin = testDown(i, j, player);
			if(isAWin) { return true; }
			else { numInARow = RESET; }
		    }
		    
		    // Tests the right-down diagonal		    
		    if((i!=this.width-1) && (j!=this.height-1)){	
			isAWin = testDownRight(i,j,player);
			if(isAWin) { return true; }
			else { numInARow = RESET; }
		    }
		
		    // Tests the horizontal		
		    if(i!=this.width-1) {	
			isAWin = testRight(i, j, player);
			if(isAWin) { return true; }
			else { numInARow = RESET; }
		    }
		}
	    }
	}
	return isAWin;
    }
    
    /************************************************************
     * boolean testDownLeft()                                   *
     * Returns true if there is a connection formed via the down*
     * left diagonal.					        *
     * Accepts coordinates and a player's character as input    *
     ************************************************************/
    public boolean testDownLeft(int x, int y, int player) {	
	boolean returnVal = false;
	if(this.matrix[x][y] != player) {
	    returnVal = false;	
	}
	else if(this.matrix[x][y] == player) {	
	    numInARow++;
	    if(numInARow == this.goal) { returnVal = true; }
	    else if((x!=0) && (y!=this.height-1)) {	
		returnVal = testDownLeft(x-1, y+1, player);	
	    }
	}
	return returnVal;
    }

    /************************************************************
     * boolean testDown()                                       *     
     * Returns true if there is a connection formed vertically. *
     * Accepts coordinates and a player's character as input    *
     ************************************************************/  
    public boolean testDown(int x, int y, int player) {	
	boolean returnVal = false;
	if(this.matrix[x][y] != player) { returnVal = false; }
	else if(this.matrix[x][y] == player) {	
	    numInARow++;
	    if(numInARow == this.goal) { returnVal = true; }
	    else if(y!=this.height-1) {	
		returnVal = testDown(x, y+1, player);	
	    }
	}
	return returnVal;
    }

    /************************************************************
     * boolean testDownRight()                                  *     
     * Returns true if there is a connection formed via the down*       
     * right diagonal.                                          *
     * Accepts coordinates and a player's character as input    *
     ************************************************************/    
    public boolean testDownRight(int x, int y, int player) {	
	boolean returnVal = false;
	if(this.matrix[x][y] != player) { returnVal = false; }
	else if(this.matrix[x][y] == player) { 
	    numInARow++; 
	    if(numInARow == this.goal) { returnVal = true; }
	    else if((x!=this.width-1) && (y!=this.height-1)) {	
		returnVal = testDownRight(x+1, y+1, player);	
	    }
	}
	return returnVal;
    }
    
    /************************************************************
     * boolean testDownLeft()                                   *     
     * Returns true if there is a connection formed via the     *       
     * horizontal axis.                                         *
     * Accepts coordinates and a player's character as input    *
     ************************************************************/    
    public boolean testRight(int x, int y, int player) {
	boolean returnVal = false; 	
	if(this.matrix[x][y] != player) { returnVal = false; }
	else if(this.matrix[x][y] == player) {	
	    numInARow++;
	    if(numInARow == this.goal) { returnVal = true; }
	    else if(x!=this.width-1) {	
		returnVal = testRight(x+1, y, player);	
	    }
	}
	return returnVal;
    }
    
	/**
	 * char[][] getBoard()
	 * Returns the matrix so the gui can display it.
	 **/
	public char[][] getBoard()
	{
		return(matrix);
	}



} // end of GameBoard class
