/******************************************************************************
 * ConnectNServer.java - a server for handling connectN client negotiations,
 * and statistics management.
 * CREATED: 05/01/2004
 * SYNOPSIS: 
 * If connectNServer is started with an even-numbered port passed as
 * the command line argument, then the server starts a StatsUpdater thread to
 * handle stats requests and updates.
 * If clinetListener is started with an odd-numbered port passed as the 
 * command line argument, then the server starts a ClientNegotiator thread to
 * handle player negotiations and game setup.
 *****************************************************************************/
import java.io.*;
import java.net.*;


class ConnectNServer {

  // main gets the port from the command line and tries to bind to
  // that port. if it can't, it increments by 2 and tries again.
  // when the port is successfully binded, main() waits for client(s) 
  // connections and passes them off to the appropriate thread.
  public static void main (String args[]) throws java.io.IOException {
      boolean isPortOpened = false;  // do we have access to the port?
      Socket clientOneSock = null;   // client socket 
      Socket clientTwoSock = null;   // client socket
      ServerSocket sock = null;      // server socket
      int portNum = -1;
      int MAXPORTS = 65536;

      if (args.length != 1) {
	  System.out.println("Usage: " +
			     "java connectNServer <port>");
	  System.exit(1);
      }
        
      // get the port passed as args[0]
      try {portNum = Integer.parseInt(args[0]); }
      catch(NumberFormatException n) {
	  System.out.println("Usage: " +
			     "java connectNServer <port>");
	  System.exit(1);
      }

      // try to bind to whatever port was passed on command line. if that
      // port is in use, increment portNum by 2 and try that port. continue
      // trying until we get out of valid port range. (MAXPORTS)
      while(isPortOpened == false) {	  
	  try {
	      // try to bind to the socket. 
	      sock = new ServerSocket(portNum);
	      isPortOpened = true;
	  } 

	  // if we get an IOException, the bind failed. increment by 2 and
	  // iterate through the while loop again.
	  catch (IOException e) {
	      System.out.println("Could not listen on port: "
				 + portNum + ", ");
	      portNum = portNum + 2; 
	      System.out.println("Trying " + portNum);
	  }

	  // if portNum is out of range, exit.
	  if(portNum > MAXPORTS) {
	      System.out.println("Could not find a free port to open.");
	      System.exit(-1);
	  }
      }

      System.out.print("Now listening at port " + portNum);

      // if portNum is Odd, we want to start a ClientNegotiator server.
      if(portNum % 2 == 1) {
	  System.out.println(" as a ClientNegotiator.");

	  // always stay alive. termination occurs from ctrl+c or the 
	  // kill command.
	  while (true) {
	      
		  try {
		  // wait for client 1 connect (player 1)
		  clientOneSock = sock.accept();
		  // wait for client 2 connect (player 2) 
		  clientTwoSock = sock.accept();
	      } 

	      // if accept failed, catch exception and exit.
	      catch (IOException e) {
		  System.out.println("Accept failed: "
				     + portNum + ", ");
		  System.exit(1);
	      }
		  if( clientOneSock.isConnected() && clientTwoSock.isConnected()){
			  // instantiate a negotiator thread and pass the connected
			  // sockets to it.
			  ClientNegotiator negotiator = 
				  new ClientNegotiator(clientOneSock, 
									   clientTwoSock);
			  
			  
			  // start the negotiator thread.
			  negotiator.start();
		  }

	  }
      }
      
      // otherwise portNum is even, and we want to start a stats server.
      else {
	  System.out.println(" as a StatsUpdater.");
	  Stats statsInfo = new Stats();
	  while(true) {
	      try {
		  // wait for client to connect for stats retrieval/update 
		  clientOneSock = sock.accept();
	      } 
	      catch (IOException e) {
		  System.out.println("Accept failed: "
				     + portNum + ", ");
		  System.exit(1);
	      }

	      // instantiate a StatsUpdater thread and pass the connected
	      // socket to it.
	      StatsUpdater updater = 
		  new StatsUpdater(clientOneSock, statsInfo);
	      updater.start();
	  }
      }
      
  }// end main method

}// end ConnectNServer class.

