/******************************************************************************
 * StatsUpdater.java - a thread that handles client requests to receive or 
I* update stats. This class is instantiated from the clientListener program.
 * The thread understands two commands: "stats receive" and "stats update"
 *****************************************************************************/
import java.io.*;
import java.net.*;
import java.util.*;

class StatsUpdater extends Thread {
    private Socket clientSock;
    private Stats statsInfo;
    private boolean inUse = false;

    // Constructor. Takes in the stats object that was instantiated in the
    // calling thread statsListener. Also takes in the socket that the
    // interested client is connected to. 
    StatsUpdater(Socket client, Stats statsObject) {
		this.clientSock = client;
		this.statsInfo = statsObject; 
    }
    
    // Thread begins
    public void run() {
	String line = "";
	boolean isSuccess = false; // default to fail

	try {	    

	    // open an input stream and output stream with the client's socket.
	    BufferedReader clientInput = new BufferedReader(
				   new InputStreamReader(
				   clientSock.getInputStream()));
	    PrintWriter clientOutput = new PrintWriter(
                                 clientSock.getOutputStream());

	    // informational message to console.
	    System.out.println("client connected to stats server.");

	    // get a line from the client. If it is "stats receive", then
	    // acknowledge and call sendStats(). if the line is "stats update"
	    // then acknowledge and call getNewStat. Anything else, complain 
	    // to the console. 
	    line = clientInput.readLine().trim();
	    if(line.equals("stats receive")) {
		// send stats
		clientOutput.println("ok");
		clientOutput.flush();

		// send the stats to the output socket.
		isSuccess = sendStats(clientOutput);

		// if isSuccess is false, something went wrong, and the 
		// stats weren't sent to the client properly. print an
		// informational error message and discconect.
		if(isSuccess == false) {
		    System.out.println("Sending stats to client failed.");
		}
	    }

	    // second valid case of command from client.
	    else if(line.equals("stats update")) {
		clientOutput.println("ok");
		clientOutput.flush();

		// get one line of stats from inputsocket.		
		isSuccess = getNewStat(clientInput);
		
		// if isSuccess is false, something went wrong, and the 
		// new stats were invalid and discarded. print an 
		// informational error message and discconect.
		if(isSuccess == false) {
		    System.out.println("Getting new stat from client failed.");
		}
	    }	    

	    // any other line received from the client is ignored. Complain
	    // to console and disconnect. 
	    else {
		System.out.println("StatsUpdater Protocol Error: client " +
				   "did not issue valid stats command.");
	    }

	    // close client socket
	    clientSock.close();
	    
	    return;
	}

	// catch IOException if readLine throws it.
	catch (IOException e) {	    
	    System.out.println("StatsUpdater Failed in client " +
			       "Negotiation: ");
	}

    } // end run() method

    // sendStats(): gets a Vector of strings from the statsInfo object.
    // each entry in the vector is a line of stats information in the
    // following format: <timestamp> <username> <num connected> <num moves>
    public synchronized boolean sendStats(PrintWriter output) {
	while(inUse) { 
	    try { wait(); }
	    catch (InterruptedException e) {}
	}
	inUse = true;
	
	String line = "";
	boolean isSuccess = false; // default to false
	int i;

	Vector entries = (Vector) statsInfo.getEntries();
	for(i = 0; i < entries.size(); i++) {
	    line = (String) entries.get(i);
	    output.println(line);
	    output.flush();
	}

	// when we've finished sending all the stats, send "done" so the
	// client knows.
	output.println("done");
	output.flush();

	inUse = false;
	notify();

	isSuccess = true;
	return isSuccess;

    } // end method sendStats

    // getNewStat(): gets a single entry from a client. Then performs a sanity
    // check to make sure there are 3 tokens on the line. If there are, a
    // timestamp is put at the beginning of the entry, and the statInsert
    // method is called to put the entry into memory and to disk.
  public synchronized boolean getNewStat(BufferedReader input) {
      boolean isSuccess = false; // default to false
      String dateString;         // holds today's date
      String newEntry;           // string received from client.
      String month; 
      String day;      
      
      try {
	  while(inUse) { 	    
	      try { wait(); }
	      catch (InterruptedException e) {}
	  }

	  inUse = true;

	  // get a line from the client.
	  newEntry = input.readLine().trim();

	  Calendar cal = new GregorianCalendar();
	  Date curDate = new Date();
	  cal.setTime(curDate);
	  int iMonth = cal.get(Calendar.MONTH);
	  String curToken = "";
	  String userName = "";

	  iMonth++;

	  if(iMonth < 10 ) month = "0" + iMonth;
	  else month = "" + iMonth;
	  if(cal.get(Calendar.DAY_OF_MONTH) < 10 ) day = "0" + 
				             cal.get(Calendar.DAY_OF_MONTH);
	  else day = "" + cal.get(Calendar.DAY_OF_MONTH);
	  dateString = month + "-" + day + "-" + cal.get(Calendar.YEAR);
	
	  newEntry = dateString + " " + newEntry;

	  // statInsert put the new entry into memory and saves to disk.
	  statsInfo.statInsert(newEntry);

      }

      // any IOException associated with readLine(), and return
      // failure.
      catch(IOException e) {
	  System.out.println("getNewStat error.");
	  isSuccess = false;
	  return isSuccess;
      }

      finally {
	  inUse = false;
	  notify();
      }

      isSuccess = true;
      return isSuccess;

  } // end method getNewStat

} // end class StatsUpdater
