/**
 * GuiWindow is the class which makes a threaded GUI window frame and
 * controls all the components of in the window.
 **/
import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.awt.geom.*;

public class GuiWindow extends Thread
{    
	private EventHandler listener;
	private ConnectN game;
	private JFrame ourMainFrame;
	private MenuComponent menuComponent;
	private ButtonPanel btnPanel;
	private DisplayPanel disPanel;
	private ChatPanel chatPanel;
	private LeverPanel leverPanel;
	private JPanel p1, p2;
	private SplashWindow splash;
	private StatsWindow stats;
	private ReplayBox replay;

	// constructor. save the game and board.
	GuiWindow(ConnectN theGame)      
	{
	    this.game = theGame;
	    this.listener = new EventHandler(game, this);
	    menuComponent = new MenuComponent(listener);
	    this.btnPanel = new ButtonPanel(game, listener);
	    this.disPanel = new DisplayPanel(game, listener);
	    this.chatPanel = new ChatPanel(listener);
	    this.leverPanel = new LeverPanel(game,listener);
	    splash = new SplashWindow(listener);
	    stats = new StatsWindow(game, listener);
	}

        // start the thread and build the gui.
        public void run()
	{

	    //Make sure we have nice window decorations.
	    JFrame.setDefaultLookAndFeelDecorated(true);
	    
	    ourMainFrame = new JFrame("ConnectN");
		ourMainFrame.setBackground(ConnectN.GUICOLOR);
		ourMainFrame.addWindowListener(listener);
	    //ourMainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	    Graphics gfx = ourMainFrame.getGraphics();
	    
	    Container frameContent = ourMainFrame.getContentPane();
	    frameContent.setLayout(new BoxLayout(frameContent, BoxLayout.Y_AXIS));
		frameContent.setBackground(ConnectN.GUICOLOR);
	    
	    //Create a panel which holds the ButtonPanel and the DisplayPanel
	    //in a vertical arrangement
	    
	    p1 = new JPanel();
		p1.setBackground(ConnectN.GUICOLOR);
	    p1.setLayout(new BoxLayout(p1, BoxLayout.Y_AXIS));	
	    
	    p1.add(btnPanel);
	    p1.add(Box.createHorizontalStrut(20));
	    p1.add(disPanel);
	    //End of p1
	    
	    //Create a panel which holds p1 and the LeverPanel in a horizontal
	    //arrangement
	    p2 = new JPanel();
		p2.setBackground(ConnectN.GUICOLOR);
	    p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
	    
	    p2.add(p1);
	    p2.add(Box.createVerticalStrut(20));
	    p2.add(leverPanel);
	    //End of p2
	    
	    frameContent.add(p2);
	    frameContent.add(Box.createHorizontalStrut(20));
	    frameContent.add(chatPanel);
	    
	    ourMainFrame.setJMenuBar(menuComponent.createMenuBar());
	    

		ourMainFrame.pack();
		ourMainFrame.setVisible(false);
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		Rectangle window = ourMainFrame.getBounds();
		ourMainFrame.setLocation(
				 ((screen.width - window.width) /2),
				 ((screen.height - window.height) /2));



	    if(game.getGameState() == game.STARTUP){
			//System.out.println("game is startup");
		splash.setVisible(true);
	    }
	    
	}
    
    /**
     * updateChatWindow - public method to append a line to the chat window 
     * pane.
     **/
    public void updateChatWindow(String line) {
	this.chatPanel.updateText(line);
    }

    /**
     * resetChatWindow - public method to clear the chat window
     **/
    public void resetChatWindow() {
	this.chatPanel.clearText();
    }

	/**
	 * Accessor for the disPanel size
	 **/
	public int getPaneSizeX(){
		return( disPanel.getWidth() );
	}

	public int getPaneSizeY(){
		return( disPanel.getHeight() );
	}

	/**
	 * getGuiFrame is used to return the frame for use with setting
	 * locations of other frames
	 **/
	public JFrame getGuiFrame(){
		return ourMainFrame;
	}

	/**
	 * updateDisplayBoard is used because the gui doesn't
	 * extend jframe
	 **/
	public void updateDisplayBoard()
	{
	    GameBoard board = game.getGameBoard();
		splash.setVisible(false);
	    this.disPanel.setDisplayBoardSize(board.getWidth(), 
					      board.getHeight());	    
	    this.disPanel.repaint();
	    handleGreyMenu();
		this.leverPanel.repaint();
	    this.btnPanel.buildButtons();
		ourMainFrame.pack();
	    ourMainFrame.show();

	}

	/**
	 * method used to dispose of the stats window
	 **/
	public void disposeStatsWindow() {
		stats.setVisible(false);
	}

	/**
	 * method used to create the stats window
	 **/
	public void showStatsWindow() {
	    // if the game was won, and i am the winner, and i haven't updated 
	    // the server before, update the stat server.
	    if( (game.getGameState() == game.WINGAME) &&
		(game.getWinner().equals(game.getUserName1())) &&
		(game.isStatSent() == false) ) {
		if( stats.sendStatToServer() ) {
		    game.setStatSent(true);
		}
	    }
	    // now build the stats window with the most recent stats.
	    if(stats.buildStatsWindow() != true ) {
		updateChatWindow("There was an error getting stats from " +
				 "the server.");
	    }
	    
	}

    public void showSaveDialogue() {
	JFileChooser choose = new JFileChooser();
	choose.setBackground(ConnectN.GUICOLOR);
	choose.setApproveButtonText("Save");
	if(choose.showOpenDialog(ourMainFrame) == JFileChooser.APPROVE_OPTION)
	    game.saveLocalGame( choose.getSelectedFile().getPath() );
    }

    public void showLoadDialogue() {
	JFileChooser choose = new JFileChooser();
	choose.setBackground(ConnectN.GUICOLOR);
	choose.setApproveButtonText("Load");
	if(choose.showOpenDialog(ourMainFrame) == JFileChooser.APPROVE_OPTION)
	    game.loadLocalGame( choose.getSelectedFile().getPath() );
    }

    /**
     * method used to grey out Load Game and/or Save game, 
     * when appropriate.
     **/
    public void handleGreyMenu() {
	boolean save = false;
	boolean load = false;
	boolean stat = false;


	// if we're online, see if we're in a game.
	if((game.getOnlineState() == game.ONLINE)) {
	    //System.out.println("online");
		if(game.getGameState() == game.INGAME) {
			//System.out.println(",ingame");
			stat = true;
			save = false;
			load = false;
		}
		// if we're not in a game, just enable load.
		else {
			//System.out.println(", either outgame or wingame");
			stat = true;
			load = true;
			save = false;	    
		}
	}
	// if we're waiting for another player, disable load and save.
	else if(game.getOnlineState() == game.WAITING) {
	    //System.out.println("waiting");
		stat = true;
	    load = false;
	    save = false;
	}
	
	// else we're offline
	else { 	    
	    // if we're offline and in a game, enable save and load.
	    if(game.getGameState() == game.INGAME) {
			//System.out.println("ingame");
			load = true;
			save = true;	    
			stat = false;
	    }
		
	    // else only enable load.
	    else {
			//System.out.println("not online, not in a game.");
			load = true;
			save = false;
			stat = false;
	    }	       	    
	}	     
	
	//System.out.println("save: " + save + " load: " + load
	//				   + "stat: " + stat );
	ourMainFrame.setJMenuBar(menuComponent.updateMenuBar(save, load, stat));
    }
    
    /** 
     * Animates the lever
     **/
    public void animateLever() {
 	leverPanel.animateLever();
    }


	/**
	 * replayGame creates a new replayGame window
	 **/
	public void replayGame(){
		replay = new ReplayBox(listener);
		replay.pack();
		replay.show();
	}

	/**
	 * replayGameKill() ends the replayGame window
	 **/
	public void replayGameKill(){
		replay.setVisible(false);
		replay.dispose();
	}

}

	



/**
 * Pops up a box after the winner closes the stats window asking to
 * replay or not
 **/
class ReplayBox extends JFrame {

	ReplayBox(EventHandler listener) {
	
		this.setTitle("Replay?");
		this.setBackground(ConnectN.GUICOLOR);
		ReplayPanel picPanel = new ReplayPanel();


		JPanel thePanel = new JPanel(new GridLayout(1,2));
		thePanel.setBackground(ConnectN.GUICOLOR);
		JButton btnAgain = new JButton("Play Again");
		btnAgain.setBackground(ConnectN.GUICOLOR);
		btnAgain.setActionCommand("playAgain");
		btnAgain.addActionListener(listener);
		
		JButton btnDisconnect = new JButton("Disconnect");
		btnDisconnect.setBackground(ConnectN.GUICOLOR);
		btnDisconnect.setActionCommand("Disconnect");
		btnDisconnect.addActionListener(listener);
		
		thePanel.add(btnDisconnect);
		thePanel.add(btnAgain);
		this.getContentPane().setLayout(new BoxLayout(this.getContentPane(),
													  BoxLayout.Y_AXIS));
		this.getContentPane().add(picPanel);
		this.getContentPane().add(thePanel);
	
    }

	class ReplayPanel extends JPanel
	{
		
		private TexturePaint imagePaint;
		private BufferedImage image;
		private String imageFile = "replay.jpg";
		
		ReplayPanel(){
			setPreferredSize(new Dimension(370, 400));
			image = ImageUtilities.getBufferedImage(imageFile, this);
			
		}
		
		public void paintComponent(Graphics g){
			Graphics2D gfx = (Graphics2D)g;
			
			Rectangle background = new Rectangle(0, 0,
												 this.getWidth(),
												 this.getHeight());
			gfx.fill(background);
			int border = 0;
			imagePaint = new TexturePaint(image,
										  new Rectangle(border, border,
										  	this.getWidth() - border*2,
											this.getHeight() -border*2)); 
			gfx.setPaint(imagePaint);
			Rectangle imageRect = new Rectangle(border, border,
										  	this.getWidth() -border*2,
										  		this.getHeight() -border*2);
			gfx.fill(imageRect);
			
			
			
		}
		
	}
}

